@@ -1,28 +1,35 @@
+//!This Crate Provides functionality, for injecting dlls into other processes.
+//!Most of the crate is right now accessible through the [Injector] class.
+//!
+//!You will need to provide a pid, and a dll to inject. This crate will do the rest for you.
+//!
+//! The main focus will always be on performing the injection reliable.
+//! If you care about injecting into a 64 bit application whilst needing to compile this library under 32 bits, you will want to enable the "x86tox64" feature.
+//! Be aware, that that feature uses "unofficial" api's located in ntdll.dll.
+//! Compatibility is technically not guaranteed by windows.
+//!
+//!If you have any suggestions, on improving the outfacing api of this crate create an issue, or pr.
+//!I am not sure yet, if I like this design.
+//!
+//!Linux support will probably not come.
+//!It is insanely hard and platform specific, because
+//! 1. we would need to write raw machinecode/shellcode to the target process.
+//! 3. which then has the necessary code to load the .so
+//! 4. we need to somehow redirect the target program's execution, to execute our code
+//! 5. we need to do that, without somehow disrupting ANY of the program's code
+//! 6. we need to return the EXACT state before we did anything, because the other program may need that
+//!
+//! If this library is supposed to be helpful I'd want to not require to run it as root.
+//! Unfortunately some steps involve calling ptrace. Access to the command is restricted, if you are not the parent process of the process you are trying to trace.
+//! These requirements would mean, that we can only inject so files to processes, that the program this library itself created.
+// #![feature(strict_provenance)]
+// #![warn(lossy_provenance_casts)]
 #![warn(missing_docs)]
-///This Crate Provides functionality, for injecting dlls into other processes.
-///Most of the crate is right now accessible through the [Injector] class.
-///
-///You will need to provide a pid, and a dll to inject. This crate will do the rest for you.
-///
-///The main focus will always be on performing the injection reliable.
-///
-///If you have any suggestions, on improving the outfacing api of this crate create an issue, or pr.
-///I am not sure yet, if I like this design.
-///
-///Linux support will probably not come.
-///It is insanely hard and platform specific, because
-/// 1. we would need to write raw machinecode/shellcode to the target process.
-/// 3. which then has the necessary code to load the .so
-/// 4. we need to somehow redirect the target program's execution, to execute our code
-/// 5. we need to do that, without somehow disrupting ANY of the program's code
-/// 6. we need to return the EXACT state before we did anything, because the other program may need that
-///
-/// If this library is supposed to be helpful I'd want to not require to run it as root.
-/// Unfortunately some steps involve calling ptrace. Access to the command is restricted, if you are not the parent process of the process you are trying to trace.
-/// These requirements would mean, that we can only inject so files to processes, that the program this library itself created.
 
 ///This struct will expose certain module private functions, to actually use the api.
+///The exact contents should be considered implementation detail.
 #[derive(Debug, Clone)]
+#[non_exhaustive]
 pub struct Injector<'a> {
     ///The path to a dll. This may be in any format, that rust understands
     pub dll: &'a str,
@@ -32,11 +39,21 @@ pub struct Injector<'a> {
 
 pub(crate) type Result<T> = std::result::Result<T, error::Error>;
 pub(crate) use log::{debug, error, info, trace, warn};
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 ///Holds all error types
 pub mod error;
 mod platforms;
+///This represents the actions, that are supported with a dll.
+pub trait Inject {
+    ///Injects a dll
+    fn inject(&self) -> Result<()>;
+    ///Ejects a dll
+    fn eject(&self) -> Result<()>;
+    ///This Function will find all currently processes, with a given name.
+    ///Even if no processes are found, an empty Vector should return.
+    fn find_pid<P: AsRef<Path>>(name: P) -> Result<Vec<u32>>;
+}
 
 impl<'a> Injector<'a> {
     ///Create a new Injector object.
@@ -51,6 +68,19 @@ impl<'a> Injector<'a> {
     pub fn set_pid(&mut self, pid: u32) {
         self.pid = pid;
     }
+    #[cfg(target_family = "windows")]
+    ///Gets the Platform specific Injector.
+    ///Currently only windows is supported.
+    ///wait indicates, if we should wait on the dll to attach to the process
+    pub fn inject(&self, wait: bool) -> impl Inject + '_ {
+        platforms::windows::InjectWin { inj: self, wait }
+    }
+    #[cfg(target_family = "windows")]
+    ///This Function will find all currently processes, with a given name.
+    ///Even if no processes are found, an empty Vector should return.
+    pub fn find_pid<P: AsRef<Path>>(name: P) -> Result<Vec<u32>> {
+        platforms::windows::InjectWin::find_pid(name)
+    }
 }
 
 impl<'a> Default for Injector<'a> {
