@@ -1,5 +1,5 @@
 #[cfg(target_os = "windows")]
-mod windows;
+pub mod windows;
 
 #[cfg(all(target_arch = "x86", feature = "x86tox64"))]
 mod x86;
