@@ -1,5 +1,5 @@
 use super::macros::err;
-use crate::error::Error;
+use crate::error::CustomError;
 use crate::Result;
 use log::{info, trace};
 use std::ops::Deref;
@@ -29,26 +29,23 @@ impl Thread {
     pub(crate) fn wait_for_thread(&self) -> Result<()> {
         info!("Waiting for thread");
         return match unsafe { WaitForSingleObject(self.thread, INFINITE) } {
-            0x80 => Err(Error::Winapi(
-                "WaitForSingleObject returned WAIT_ABANDONED".to_string(),
-                0x80,
-            )), //WAIT_ABANDONED
+            //             0x80 => Err(Error::Winapi(
+            //                "WaitForSingleObject returned WAIT_ABANDONED".to_string(),
+            //                0x80,
+            //            )), //WAIT_ABANDONED
             0x0 => {
                 info!("Dll eject success? IDK?! Hopefully? WaitForSingleObject returned WAIT_OBJECT_0");
                 Ok(())
             } //WAIT_OBJECT_0
-            0x102 => Err(Error::Winapi(
-                "Timeout hit at WaitForSingleObject.".to_string(),
-                0x102,
-            )), //WAIT_TIMEOUT
-            0xFFFFFFFF => Err(Error::Winapi(
-                "Wait_Failed hit at WaitForSingleObject.".to_string(),
-                0xFFFFFFFF,
-            )), //WAIT_FAILED
-            e => Err(Error::Unsuccessful(Some(format!(
-                "WaitForSingleObject return code {}",
-                e
-            )))),
+            //            0x102 => Err(Error::Winapi(
+            //                "Timeout hit at WaitForSingleObject.".to_string(),
+            //                0x102,
+            //            )), //WAIT_TIMEOUT
+            //            0xFFFFFFFF => Err(Error::Winapi(
+            //                "Wait_Failed hit at WaitForSingleObject.".to_string(),
+            //                0xFFFFFFFF,
+            //            )), //WAIT_FAILED
+            e => Err(CustomError::WaitForSingleObject(e))?,
         };
     }
 }
